package com.docus.server.infrastructure.dao.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.docus.core.util.Func;
import com.docus.infrastructure.core.db.dao.impl.BaseDaoImpl;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.entity.scheduling.management.SchCollectorVersion;
import com.docus.server.infrastructure.dao.ISchCollectorVersionDao;
import com.docus.server.infrastructure.mapper.SchCollectorVersionMapper;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;

/**
 * 采集器版本列表管理 数据访问实现类
 *
 * @author AutoGenerator
 * @since 2023-07-15
 */
@Repository
public class SchCollectorVersionDaoImpl extends BaseDaoImpl<SchCollectorVersionMapper, SchCollectorVersion> implements ISchCollectorVersionDao {

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public SchCollectorVersion findById(String id) {
        return super.findById(id);
    }

    /**
     * 新增
     *
     * @param schCollectorVersion 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(SchCollectorVersion schCollectorVersion) {
        return super.save(schCollectorVersion);
    }

    /**
     * 编辑
     *
     * @param schCollectorVersion 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(SchCollectorVersion schCollectorVersion) {
        return super.updateById(schCollectorVersion);
    }

    /**
     * 批量删除
     *
     * @param ids 主键ids
     * @return 成功或失败
     */
    @Override
    public int delete(List<Long> ids) {
        return super.deleteByIds(ids);
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<SchCollectorVersion> search(SearchDTO searchDTO) {
        //分页
        PageHelper.startPage(searchDTO.getPageNum(), searchDTO.getPageSize());
        LambdaQueryWrapper<SchCollectorVersion> query = Wrappers.lambdaQuery();
        //时间范围，默认按createTime
        if (searchDTO.getBeginTime() != null) {
            query.ge(SchCollectorVersion::getCreateTime, searchDTO.getBeginTime());
        }
        if (searchDTO.getEndTime() != null) {
            Date endTime = searchDTO.getEndTime();
            query.le(SchCollectorVersion::getCreateTime, endTime);
        }

        Object collectorId = searchDTO.getParams("collectorId");
        if (Func.isNotEmpty(collectorId)) {
            query.eq(SchCollectorVersion::getCollectorId, collectorId);
        }

        //默认createTime倒序排序
        query.orderByDesc(SchCollectorVersion::getCreateTime);
        List<SchCollectorVersion> list = super.find(query);
        return new PageResult<>(list);
    }

    @Override
    public SchCollectorVersion findByVersion(Long collectorId, String collectorVersionNo) {
        LambdaQueryWrapper<SchCollectorVersion> query = Wrappers.lambdaQuery();

        if (Func.isNotEmpty(collectorId)) {
            query.ge(SchCollectorVersion::getCollectorId, collectorId);
        }
        if (Func.isNotBlank(collectorVersionNo)) {
            query.le(SchCollectorVersion::getCollectVersion, collectorVersionNo);
        }

        return super.findOne(query);

    }

}
