package com.docus.server.sys.infrastructure.mapper;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.docus.server.constants.DSKeyConstants;
import com.docus.server.sys.common.pojo.dto.UserModifyParam;
import com.docus.server.sys.common.pojo.entity.PowerUser;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * <p>
 * 用户表 baseMapper
 * </p>
 *
 * @author wen yongbin
 * @since 2023年2月25日22:28:58
 */
@Mapper
@DS(DSKeyConstants.SYSTEM_DS_KEY)
public interface PowerUserMapper extends BaseMapper<PowerUser> {

    /**
     * 根据用户工号查询用户信息
     *
     * @param userName 用户工号
     * @return 用户信息
     */
    PowerUser getUserByUserName(@Param("userName") String userName);

    /**
     * 更新用户信息
     *
     * @param userModifyParam 用户操作参数
     * @return 数据库更新信息
     */
    int updateUser(@Param("user") UserModifyParam userModifyParam);

    /**
     * 添加新用户
     *
     * @param userModifyParam 用户操作参数
     * @return 数据库添加信息
     */
    int addUser(@Param("user") UserModifyParam userModifyParam);

    /**
     * 根据用户工号删除用户信息
     *
     * @param userName 用户工号
     * @return 数据库删除信息
     */
    int delUserByUserName(@Param("userName") String userName);
}
